//
//  main.h
//  BulletTest
//
//  Copyright (c) 2011 Apple Inc.
//

#ifndef BulletTest_main_h
#define BulletTest_main_h

#ifdef __cplusplus
extern "C" { 
#endif

    extern int gReportAverageTimes;     // if 0, report best times
    extern int gExitOnError;            // if non-zero, exit as soon an an error is encountered
    extern const char *gAppName;        // the name of this application
    
#ifdef __cplusplus
}
#endif



#endif
